/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 18.09.19 -
 */

#ifndef INC_4NEURO_LAZYLEARNING_H
#define INC_4NEURO_LAZYLEARNING_H

#include <memory>
#include "../settings.h"
#include "../constants.h"
#include "LearningMethod.h"
#include "../ErrorFunction/ErrorFunctions.h"

namespace lib4neuro {
    /**
     *
     */
    class LazyLearning : public LearningMethod {

    private:
		size_t subset_size;

		LearningMethod *inner_method;

		double tolerance;

    public:

        /**
         * Creates an instance of Lazy Learning Optimizer
		 * it trains the network iteratively using chunks of the train data which cause
		 * the highest error
         * @param inner_trainer method used for individual optimizations
         * @param subset_size number of data entries to be propagated in one batch
         * @param tolerance desired accuracy
         */
        LIB4NEURO_API explicit LazyLearning(
			LearningMethod &inner_trainer,
			double tol = 1e-6
		);

        /**
         * Deallocates the instance
         */
        LIB4NEURO_API ~LazyLearning();

        /**
         *
         * @param ef
         */
        LIB4NEURO_API void optimize(lib4neuro::ErrorFunction& ef,
                                    std::ofstream* ofs = nullptr) override;

    };
}

#endif //INC_4NEURO_GRADIENTDESCENT_H
