//
// Created by martin on 05.09.19.
//

#ifndef LIB4NEURO_NELDERMEAD_H
#define LIB4NEURO_NELDERMEAD_H

#include <vector>

#include "LearningMethod.h"

/* Forward declaration or arma::Mat<> type */
namespace arma {
    template<class T>
    class Mat;
}

namespace lib4neuro {
    class NelderMead : public LearningMethod {
    protected:
        /**
         * Maximal number of iterations - optimization will stop after that, even if not converged
         */
        unsigned int maximum_niters;

        double reflection_coefficient;

        double expansion_coefficient;

        double contraction_coefficient;

        double shrink_coefficient;

        double terminating_net_error;

        double terminating_vertex_difference;

        double terminating_simplex_contraction;

        /**
         * Initial size of the simplexes' edges
         */
         double initial_edge_size;

         /**
          *
          */
         unsigned int optimized_function_dimension;

         /**
          * Matrix containing info about the currently used simplex. Every column corresponds with coordinates
          * of a single vertex.
          */
        arma::Mat<double>* simplex;

         /**
          * Computes a contraction measure of the simplex according to
          *
          * max(||x_i - x_1||) / max(1, ||x_1|| ) <= contraction_threshold
          *
          * @return Contraction measure
          */
         double get_simplex_contraction();

    public:
        LIB4NEURO_API NelderMead(double reflectionCoefficient,
                                 double expansionCoefficient,
                                 double contractionCoefficient,
                                 double shrinkCoefficient,
                                 long long int maximumNiters,
                                 double terminating_net_error,
                                 double terminating_vertex_difference,
                                 double terminating_simplex_contraction,
                                 double initial_edge_size);

        LIB4NEURO_API NelderMead(long long int maximumNiters,
                                 double initial_edge_size);

        LIB4NEURO_API explicit NelderMead(double initial_edge_size);

        LIB4NEURO_API virtual ~NelderMead();

        // TODO rewrite method names to snake case
        LIB4NEURO_API [[nodiscard]] long long int getMaximumNiters() const;

        LIB4NEURO_API [[nodiscard]] double getReflectionCoefficient() const;

        LIB4NEURO_API [[nodiscard]] double getExpansionCoefficient() const;

        LIB4NEURO_API [[nodiscard]] double getContractionCoefficient() const;

        LIB4NEURO_API [[nodiscard]] double getShrinkCoefficient() const;

        LIB4NEURO_API void optimize(lib4neuro::ErrorFunction& ef,
                                    std::ofstream* ofs = nullptr) override;

        [[nodiscard]] double getTerminatingNetError() const;

        [[nodiscard]] double getTerminatingVertexDifference() const;

        [[nodiscard]] double getTerminatingSimplexContraction() const;
    };
}

#endif //LIB4NEURO_NELDERMEAD_H
