/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 12.10.19 -
 */

#ifndef INC_4NEURO_NORMALIZEDGRADIENTDESCENT_H
#define INC_4NEURO_NORMALIZEDGRADIENTDESCENT_H

#include "../settings.h"
#include "../constants.h"
#include "LearningMethod.h"
#include "../ErrorFunction/ErrorFunctions.h"

namespace lib4neuro {
    /**
     *
     */
    class NormalizedGradientDescent : public GradientLearningMethod {

    private:

        /**
         * Threshold for the successful ending of the optimization - deviation from minima
         */
        double tolerance;

        /**
         *
         */
        size_t batch;

        /**
         * Maximal number of iterations - optimization will stop after that, even if not converged
         */
        long long int maximum_niters;

        

    public:

        /**
         * Creates an instance of Gradient Descent Optimizer (i.e. back-propagation)
         * @param epsilon Threshold for the successful ending of the optimization - deviation from minima
         * @param max_iters Maximal number of iterations - optimization will stop after that, even if not converged
         */
        LIB4NEURO_API explicit NormalizedGradientDescent(
			double epsilon = 1e-3,
            int max_iters = 1000,
            size_t batch = 0
		);

        /**
         * Deallocates the instance
         */
        LIB4NEURO_API ~NormalizedGradientDescent();

        /**
         *
         * @param ef
         */
        LIB4NEURO_API void optimize(lib4neuro::ErrorFunction& ef,
                                    std::ofstream* ofs = nullptr) override;

    };
}

#endif //INC_4NEURO_NORMALIZEDGRADIENTDESCENT_H
