/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 13.10.19 -
 */

#ifndef INC_4NEURO_SMOOTHINGLEARNING_H
#define INC_4NEURO_SMOOTHINGLEARNING_H

#include <memory>
#include "../settings.h"
#include "../constants.h"
#include "LearningMethod.h"
#include "../ErrorFunction/ErrorFunctions.h"

namespace lib4neuro {
    /**
     *
     */
    class SmoothingLearning : public LearningMethod {

    private:

        /*
        method used for the actual learning
        */
		LearningMethod *inner_method;

        /*
        accuracy of the outer loop
        */
		double learning_tolerance;

		/*

		*/
		double growth_rate;

    public:

        /**
         * Creates an instance of Lazy Learning Optimizer
		 * it trains the network iteratively using chunks of the train data which cause
		 * the highest error
         * @param inner_trainer method used for individual optimizations
         * @param subset_size number of data entries to be propagated in one batch
         * @param tolerance desired accuracy
         */
        LIB4NEURO_API explicit SmoothingLearning(
			LearningMethod &inner_trainer,
			double tol = 1e-6,
			double grate = 0.05
		);

        /**
         * Deallocates the instance
         */
        LIB4NEURO_API ~SmoothingLearning();

        /**
         *
         * @param ef
         */
        LIB4NEURO_API void optimize(lib4neuro::ErrorFunction& ef,
                                    std::ofstream* ofs = nullptr) override;

    };
}

#endif //INC_4NEURO_SMOOTHINGLEARNING_H
