/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 18.7.18 -
 */

#ifndef INC_4NEURO_NEURALNETWORKSUM_H
#define INC_4NEURO_NEURALNETWORKSUM_H

#include "../settings.h"
#include "../General/ExprtkWrapper.h"

#include "NeuralNetwork.h"

namespace lib4neuro {

    class NeuralNetworkSum : public NeuralNetwork {
    private:
        std::vector<NeuralNetwork*>* summand;
        std::vector<ExprtkWrapper*>* summand_coefficient;


    public:
        /**
         * Struct used to access private properties from
         * the serialization function
         */
        struct access;

        LIB4NEURO_API NeuralNetworkSum();

        LIB4NEURO_API virtual ~NeuralNetworkSum();

        LIB4NEURO_API void add_network(NeuralNetwork* net,
                                       std::string expression_string);

        /**
         *
         * @param input
         * @param output
         * @param custom_weights_and_biases
         */
        LIB4NEURO_API void eval_single(std::vector<double>& input,
                                       std::vector<double>& output,
                                       std::vector<double>* custom_weights_and_biases = nullptr) override;

        /**
         *
         * @param input
         * @param output
         * @param custom_weights_and_biases
         */
        LIB4NEURO_API void eval_single_debug(std::vector<double>& input,
                                             std::vector<double>& output,
                                             std::vector<double>* custom_weights_and_biases = nullptr) override;


        /**
         *
         * @param error_derivative
         * @param gradient
         */
        LIB4NEURO_API void
        add_to_gradient_single(std::vector<double>& input,
                               std::vector<double>& error_derivative,
                               double error_scaling,
                               std::vector<double>& gradient) override;

        /**
         *
         * @return
         */
        LIB4NEURO_API size_t get_n_inputs() override;

        /**
         *
         * @return
         */
        LIB4NEURO_API size_t get_n_outputs() override;

        /**
         *
         * @return
         */
        LIB4NEURO_API size_t get_n_weights() override;

        /**
         *
         * @return
         */
        LIB4NEURO_API size_t get_n_biases() override;

        /**
         *
         * @return
         */
        LIB4NEURO_API size_t get_n_neurons() override;
        /**
         *
         * @return
         */
        //TODO only works if all the networks share the same parameters
        LIB4NEURO_API std::vector<double>* get_parameter_ptr_weights() override;

        /**
         *
         * @return
         */
        //TODO only works if all the networks share the same parameters
        LIB4NEURO_API std::vector<double>* get_parameter_ptr_biases() override;
    };

}

#endif //INC_4NEURO_NEURALNETWORKSUM_H
