/**
 * DESCRIPTION OF THE CLASS
 *
 * @author Martin Beseda
 * @author Martin Mrovec
 * @author Michal Kravčenko
 * @date 2017 - 2019
 */

#ifndef INC_4NEURO_NEURONBINARYBIASED_H
#define INC_4NEURO_NEURONBINARYBIASED_H

#include "NeuronBiased.h"

namespace lib4neuro {

/**
 *  BinaryBiased neuron class - uses unit-step as the activation function
 */
    class NeuronBinaryBiased : public NeuronBiased {

    public:

        /**
         * Struct used to access private properties from
         * the serialization function
         */
        struct access;

        /**
         * Default constructor for the BinaryBiased Neuron
         * @param[in] threshold Denotes when the neuron is activated
         * When neuron potential exceeds 'threshold' value it becomes excited
         */
        LIB4NEURO_API explicit NeuronBinaryBiased(double b = 0.0);
        

        /**
         * Performs the activation function and stores the result into the 'state' property
         */
        LIB4NEURO_API double activate(double x,
                                      double b) override;
                                      
        LIB4NEURO_API double activation_function_eval_derivative_bias(double x,
                                                                      double b) override;

        LIB4NEURO_API double activation_function_eval_derivative(double x,
                                                                 double b) override;

        LIB4NEURO_API Neuron* get_derivative() override;

    };

}

#endif //INC_4NEURO_NEURONBINARYBIASED_H
