//
// Created by martin on 16.08.19.
//

#include <algorithm>
#include <cctype>

#include "../mpi_wrapper.h"

#include "../exceptions.h"
#include "Reader.h"

lib4neuro::Reader::Reader(const std::string& file_path, bool ignore_first_line, std::string delimiter) {
    this->file_path = file_path;
    this->ignore_first_line = ignore_first_line;
    this->delimiter = delimiter;
    this->data_set = nullptr;
}

std::vector<std::vector<std::string>>* lib4neuro::Reader::get_data() {
    return &this->data;
}

void lib4neuro::Reader::print_data() {
    if( lib4neuro::mpi_rank > 0 ){
        return;
    }

    for (auto line : this->data) {
        for (auto e : line) {
            std::cout << e << " ";
        }
        std::cout << std::endl;
    }
}

void lib4neuro::Reader::read() {
    // TODO make filepath relative to an executable directory instead of the current one!
    // TODO https://www.boost.org/doc/libs/1_36_0/libs/filesystem/doc/reference.html#initial_path
    if( lib4neuro::mpi_rank == 0 ){
        std::ifstream ifs(this->file_path);

        if ( !ifs.is_open() ) {
            // throw std::logic_error("File could not be opened! Possibly due to the lack of access privileges or an incorrect path.\n");
            MPI_INTERRUPT
            THROW_LOGIC_ERROR( "File could not be opened! Possibly due to the lack of access privileges or an incorrect path.\n" );
        }

        std::string   line;

        if (this->ignore_first_line) {
            std::getline(ifs,
                         line);
        }

        /* Read single line from the file */
        while (std::getline(ifs,
                            line)) {

            /* Ignore empty line */
            if (line == "") {
                continue;
            }

            /* Separate elements of the line according to the delimiter */
            size_t                   last = 0;
            size_t                   next = 0;
            std::vector<std::string> separated_line;
            while ((next = line.find(this->delimiter,
                                     last)) != std::string::npos) {
                separated_line.emplace_back(line.substr(last,
                                                        next - last));
                last = next + 1;
            }
            separated_line.emplace_back(line.substr(last));

            /* Store the elements from the line to the vector with data */
            this->data.emplace_back(separated_line);
        }

        ifs.close();
    }
    MPI_ERROR_CHECK
}

void lib4neuro::Reader::remove_white_characters_from_vector(std::vector<std::string>& v) {
    for(auto& e : v) {
        e.erase(std::remove_if(e.begin(), e.end(), [](int i){return std::isspace(i);}), e.end());
    }
    v.erase(std::remove(v.begin(), v.end(), ""), v.end());
}
