//
// Created by martin on 16.08.19.
//

#ifndef LIB4NEURO_XYZREADER_H
#define LIB4NEURO_XYZREADER_H

#include <string>
#include <vector>
#include <memory>
#include <algorithm>
#include <unordered_map>

#include "../settings.h"
#include "../DataSet/DataSet.h"
#include "../SymmetryFunction/Element.h"
#include "Reader.h"

namespace lib4neuro {
    class XYZReader : public Reader {
    protected:
        int charge;

        int ignore_charge;

        std::shared_ptr<DataSet> acsf_data_set;

        std::shared_ptr<std::vector<ELEMENT_SYMBOL>> element_list;

        std::unordered_map<ELEMENT_SYMBOL, unsigned int> n_particles_per_element;

        LIB4NEURO_API void transform_input_to_acsf(std::unordered_map<ELEMENT_SYMBOL, Element*>& element_description);

        [[nodiscard]] ELEMENT_SYMBOL get_element_symbol(std::string symbol);

    public:
        LIB4NEURO_API explicit XYZReader(const std::string& file_path, bool ignore_charge=false);

        /**
         *
         * @return
         */
        LIB4NEURO_API [[nodiscard]] std::shared_ptr<DataSet> get_data_set();

        LIB4NEURO_API [[nodiscard]] std::shared_ptr<DataSet> get_acsf_data_set();

        LIB4NEURO_API [[nodiscard]] std::shared_ptr<DataSet> get_acsf_data_set(std::unordered_map<ELEMENT_SYMBOL, Element*>& element_description);

        LIB4NEURO_API [[nodiscard]] std::shared_ptr<std::vector<ELEMENT_SYMBOL>> get_element_list();

        LIB4NEURO_API [[nodiscard]] bool contains_charge();

        LIB4NEURO_API [[nodiscard]] bool get_ignore_charge();
    };
}

#endif //LIB4NEURO_XYZREADER_H
