//
// Created by martin on 02.08.19.
//

#ifndef LIB4NEURO_SYMMETRYFUNCTION_H
#define LIB4NEURO_SYMMETRYFUNCTION_H

#include <unordered_map>
#include <vector>
#include "../settings.h"

namespace lib4neuro {

    enum class ELEMENT_SYMBOL {
        H,He,Li,Be,B,C,N,O,F,Ne,Na,Mg,Al,Si,P,S,Cl,Ar,K,Ca,Sc,Ti,V,Cr,Mn,Fe,Co,Ni,Cu,Zn,Ga,Ge,As,Se,Br,Kr,Rb,Sr,Y,Zr,
        Nb,Mo,Tc,Ru,Rh,Pd,Ag,Cd,In,Sn,Sb,Te,I,Xe,Cs,Ba,La,Ce,Pr,Nd,Pm,Sm,Eu,Gd,Tb,Dy,Ho,Er,Tm,Yb,Lu,Hf,Ta,W,Re,Os,Ir,
        Pt,Au,Hg,Tl,Pb,Bi,Po,At,Rn,Fr,Ra,Ac,Th,Pa,U,Np,Pu,Am,Cm,Bk,Cf,Es,Fm,Md,No,Lr,Rf,Db,Sg,Bh,Hs,Mt,Ds,Rg,Cn,Nh,
        Fl,Mc,Lv,Ts,Og
    };

    enum class SYMMETRY_FUNCTION_PARAMETER {
        //radius, // TODO make available also fitting of cutoff functions
        EXTENSION,
        SHIFT,
        ANGULAR_RESOLUTION,
        SHIFT_MAX,
        PERIOD_LENGTH
    };

//    enum class SYMMETRY_FUNCTION_TYPE {
//        G1, G2, G3, G4, G5
//    };

    class CutoffFunction {
    public:
        LIB4NEURO_API explicit CutoffFunction(double radius);

        /**
         * Constructor for Boost serialization.
         */
        CutoffFunction();

        LIB4NEURO_API virtual double eval(double ri) = 0;

        LIB4NEURO_API double get_radius();

        struct access;

    protected:
        double radius;
    };

    class CutoffFunction1 : public CutoffFunction {
    public:
        LIB4NEURO_API explicit CutoffFunction1(double radius);

        /**
         * Constructor for Boost serialization.
         */
        CutoffFunction1();

        LIB4NEURO_API double eval(double ri) override;

        struct access;
    };

    class CutoffFunction2 : public CutoffFunction {
    public:
        LIB4NEURO_API explicit CutoffFunction2(double radius);

        /**
         * Constructor for Boost serialization.
         */
        CutoffFunction2();

        LIB4NEURO_API double eval(double ri) override;

        struct access;
    };

    class SymmetryFunction {
    public:
//        LIB4NEURO_API explicit SymmetryFunction();

        LIB4NEURO_API explicit SymmetryFunction(CutoffFunction* cutoff_func);

        /**
         * Constructor for Boost serialization.
         */
        SymmetryFunction();

        LIB4NEURO_API virtual double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) = 0;

        LIB4NEURO_API virtual double eval(unsigned int particle_ind,
                                          std::vector<std::pair<ELEMENT_SYMBOL,
                                                                std::vector<double>>>& cartesian_coord) = 0;

        LIB4NEURO_API std::unordered_map<SYMMETRY_FUNCTION_PARAMETER, double>* get_parameters();

        LIB4NEURO_API void set_parameter(SYMMETRY_FUNCTION_PARAMETER parameter, double value);

        LIB4NEURO_API void revert_parameter_change(SYMMETRY_FUNCTION_PARAMETER parameter);

//        LIB4NEURO_API SYMMETRY_FUNCTION_TYPE get_type();

		LIB4NEURO_API void print_parameters( );

        struct access;

    protected:
//        std::vector<SYMMETRY_FUNCTION_PARAMETER> parameters;

        CutoffFunction* cutoff_func;

        std::unordered_map<SYMMETRY_FUNCTION_PARAMETER, double> parameters;

        std::unordered_map<SYMMETRY_FUNCTION_PARAMETER, double> previous_parameters;

//        SYMMETRY_FUNCTION_TYPE type;

        double euclid_distance(std::vector<double> v1, std::vector<double> v2);
		
    };

    class G1 : public SymmetryFunction {
    public:
        LIB4NEURO_API explicit G1(CutoffFunction* cutoff_func);

        /**
         * Constructor for Boost serialization.
         */
        G1();

        LIB4NEURO_API double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) override;

        LIB4NEURO_API double eval(unsigned int particle_ind,
                                  std::vector<std::pair<ELEMENT_SYMBOL,
                                                        std::vector<double>>>& cartesian_coord) override;

        struct access;
    };

    class G2 : public SymmetryFunction {
    public:
        LIB4NEURO_API explicit G2(CutoffFunction* cutoff_func, double extension, double shift);

        /**
         * Constructor for Boost serialization.
         */
        G2();

        LIB4NEURO_API double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) override;

        LIB4NEURO_API double eval(unsigned int particle_ind,
                                  std::vector<std::pair<ELEMENT_SYMBOL,
                                      std::vector<double>>>& cartesian_coord) override;

        struct access;

//    protected:
//        double extension;
//        double shift;
    };

    class G3 : public SymmetryFunction {
    public:
        LIB4NEURO_API explicit G3(CutoffFunction* cutoff_func, double period_length);

        /**
         * Constructor for Boost serialization.
         */
        G3();

        LIB4NEURO_API double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) override;

        LIB4NEURO_API double eval(unsigned int particle_ind,
                                  std::vector<std::pair<ELEMENT_SYMBOL,
                                      std::vector<double>>>& cartesian_coord) override;

        struct access;

    };

    class G4 : public SymmetryFunction {
    public:
        LIB4NEURO_API explicit G4(CutoffFunction* cutoff_func,
                                  double extension,
                                  int shift_max,
                                  double angular_resolution);

        /**
         * Constructor for Boost serialization.
         */
        G4();

        LIB4NEURO_API double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) override;

        LIB4NEURO_API double eval(unsigned int particle_ind,
                                  std::vector<std::pair<ELEMENT_SYMBOL,
                                      std::vector<double>>>& cartesian_coord) override;

        struct access;
    };

    class G5 : public SymmetryFunction {
    public:
        LIB4NEURO_API explicit G5(CutoffFunction* cutoff_func,
                                  double extension,
                                  bool shift_max,
                                  double angular_resolution);

        /**
         * Constructor for Boost serialization.
         */
        G5();

        LIB4NEURO_API double eval(unsigned int particle_ind, std::vector<std::vector<double>> cartesian_coord) override;

        LIB4NEURO_API double eval(unsigned int particle_ind,
                                  std::vector<std::pair<ELEMENT_SYMBOL,
                                      std::vector<double>>>& cartesian_coord) override;

        struct access;

    };
}

#endif //LIB4NEURO_SYMMETRYFUNCTION_H
